import 'package:dart_redis_parser/dart_redis_parser.dart';
import 'package:spec/spec.dart';

class _WritableTestCase {
  _WritableTestCase(this.command, this.arguments, this.expected);
  final String expected;
  final String command;
  final List<Object?>? arguments;
}

final _tests = <_WritableTestCase>[
  _WritableTestCase(
    'PING',
    null,
    '*1\r\n\$4\r\nPING\r\n',
  ),
  _WritableTestCase(
    'INCR',
    ['moo'],
    '*2\r\n\$4\r\nINCR\r\n\$3\r\nmoo\r\n',
  ),
  _WritableTestCase(
    'INCR',
    [true],
    '*2\r\n\$4\r\nINCR\r\n\$4\r\ntrue\r\n',
  ),
  _WritableTestCase(
    'INCR',
    ['moo', 1],
    '*3\r\n\$4\r\nINCR\r\n\$3\r\nmoo\r\n\$1\r\n1\r\n',
  ),
  _WritableTestCase(
    'INCR',
    ['moo', 6],
    '*3\r\n\$4\r\nINCR\r\n\$3\r\nmoo\r\n\$1\r\n6\r\n',
  ),
  _WritableTestCase(
    'PING',
    ['moo', 0, 1],
    '*4\r\n\$4\r\nPING\r\n\$3\r\nmoo\r\n\$1\r\n0\r\n\$1\r\n1\r\n',
  ),
  _WritableTestCase(
    'PING',
    ['moo', 0, 1, null],
    '*5\r\n\$4\r\nPING\r\n\$3\r\nmoo\r\n\$1\r\n0\r\n\$1\r\n1\r\n\$4\r\nnull\r\n',
  ),
];

void main() {
  group('writable', () {
    test('toCommandWritable should correctly return writable string values',
        () {
      for (final item in _tests) {
        expect(toCommandWritable(item.command, item.arguments))
            .toEqual(item.expected);
      }
    });

    test('fromCommandWritable should correctly return list of strings', () {
      for (final item in _tests) {
        expect(
          fromCommandWritable(item.expected),
        ).toEqual([
          item.command,
          ...?item.arguments?.map((e) => e.toString()).toList()
        ]);
      }
    });
  });
}
